    
__all__ = [
    "__title__", "__summary__", "__uri__", "__version__", "__author__",
    "__email__", "__license__", "__copyright__",
]

__title__ = "pwrusb"
__summary__ = "A python wrapper for controlling pwrusb.com power strip outlets."
__uri__ = "https://github.com/henryroe/pwrusb"

# VERSION should be PEP386 compatible (http://www.python.org/dev/peps/pep-0386)
# pre-release of a version is, e.g. 0.2.1.dev1  (0.2.1 is *newer* than 0.2.1.dev1)
# post-release of a version is, e.g. 0.2.1-1  (0.2.1 is *older* than 0.2.1-1)
__version__ = "1.0.0-2"

__author__ = "Henry Roe"
__email__ = "hroe@hroe.me"

__license__ = "MIT License"
__copyright__ = "2016 %s" % __author__



import pwrusb_swig_interface as __pwrusb

__version__ = __pwrusb.version()

class pwrusbError(Exception):
    pass

# hot-plugging/unplugging the pwrusb strips is unreliable without restarting this python package, 
# so check number of strips attached once at the beginning
n_banks = __pwrusb.get_number_of_strips_attached() 

def _validate_outlet_and_bank(outlet, bank):
    if n_banks < 1:
        raise pwrusbError("no pwrusb.com strips detected on USB")
    if outlet not in [1,2,3]:
        raise pwrusbError(("Unrecognized outlet number {0} " + 
                           "(outlet should be one of 1,2,3)").format(outlet))
    if bank > (n_banks - 1):
        raise pwrusbError(("Request bank number {0} is greater than the available " + 
                           "number of banks ({1}) counting from zero").format(bank, n_banks))

def get_outlet_state(outlet, bank=0):
    """
    Get state of single outlet on a pwrusb.com power strip.
    If only one strip attached via usb, then default bank=0 is correct.
    
    outlet = 1, 2, or 3
    
    return is True/False (on/off)
    
    (If multiple strips attached, then use bank= to access the different
    strips, but the behavior and repeatability of multiple pwrusb.com 
    strips has not been thoroughly tested.)
    """
    _validate_outlet_and_bank(outlet, bank)
    state = __pwrusb.get_single_outlet_state(bank, outlet)
    if state == 0:
        return False
    if state == 1:
        return True
    raise pwrusbError(("Unrecognized response {0} from pwrusb for (bank,outlet)=" +
                      "({1},{2})").format(state, bank, outlet))

def get_all_outlet_states(bank=0):
    """
    Get state of all outlets on a pwrusb.com power strip
    If only one strip attached via usb, then default bank=0 is correct.
        
    return is list of len=3 of True/False (on/off), e.g.:   [True, False, False]
    where outlets 1-3 are listed in order
    
    (If multiple strips attached, then use bank= to access the different
    strips, but the behavior and repeatability of multiple pwrusb.com 
    strips has not been thoroughly tested.)
    """
    _validate_outlet_and_bank(1, bank)
    states = []
    for outlet in [1, 2, 3]:
        state = __pwrusb.get_single_outlet_state(bank, outlet)
        if state == 0:
            states.append(False)
        elif state == 1:
            states.append(True)
        else:
            raise pwrusbError(("Unrecognized response {0} from pwrusb for (bank,outlet)=" +
                              "({1},{2})").format(state, bank, outlet))    
    return states

def set_outlet_state(outlet, state, bank=0):
    """
    Turn on/off outlet on a pwrusb.com power strip
    If only one strip attached via usb, then default bank=0 is correct.

    outlet = 1, 2, or 3
    state = True/False (on/off)
    
    (If multiple strips attached, then use bank= to access the different
    strips, but the behavior and repeatability of multiple pwrusb.com 
    strips has not been thoroughly tested.)
    """
    _validate_outlet_and_bank(outlet, bank)
    __pwrusb.set_single_outlet_state(bank, outlet, state)

def set_all_outlet_states(states, bank=0):
    """
    Turn on/off all outlets on a pwrusb.com power strip
    If only one strip attached via usb, then default bank=0 is correct.

    states = True -> turn all outlets on
             False -> turn all outlets off
             [True, False, True] -> turn some outlets on & some off

    (If multiple strips attached, then use bank= to access the different
    strips, but the behavior and repeatability of multiple pwrusb.com 
    strips has not been thoroughly tested.)
    """
    if states is True:
        states = [True]*3
    if states is False:
        states = [False]*3
    if len(states) != 3:
        raise pwrusbError("states must be True, False, or iterable of len=3, e.g. [True, False, True]")
    states = [True if a else False for a in states]
    for i,cur_state in enumerate(states):
        set_outlet_state(i+1, cur_state, bank=bank)

def get_total_current_milliamps(bank=0):
    """
    Read the total current (milliamps) being drawn on a pwrusb.com power strip.
    Note that will not be 0 even when all outlets are off as the strip itself draws a 
    small amount of current.
    
    There is no way to read the individual outlet current draw.
    """
    return __pwrusb.get_total_current_milliamps(bank)
    
    


from distutils.core import setup
from distutils.command.install import install
from distutils.command.build import build
from distutils.sysconfig import get_python_lib
import subprocess
import shutil
import platform
from codecs import open  # To use a consistent encoding
import os
import subprocess
import re

class SetupError(Exception):
    pass

if platform.system() != 'Darwin':
    raise SetupError('pwrusb package is designed for OSX and will not run on other systems.')

if shutil.which('swig') is None:
    raise SetupError('swig must be installed, e.g. with:\n\tbrew install swig')

for cmd in ['g++', 'ld', 'install_name_tool']:
    if shutil.which(cmd) is None:
        raise SetupError(('unable to find {}; probably means you need to install ' + 
                          'Xcode Command Line Utilities').format(cmd))

usb_lib = 'usb-1.0'
strip_whitespace = lambda s: re.sub('[\s+]', '', s)
if (strip_whitespace('ld: library not found for -l{0}'.format(usb_lib)) in 
    strip_whitespace(subprocess.run("ld -l{0}".format(usb_lib), shell=True, 
                                    stderr=subprocess.PIPE).stderr.decode("utf-8"))):
    raise SetupError(("USB library {0} not found, consider installing with, e.g.:\n\t" +
                      "brew install libusb").format(usb_lib))

base_dir = os.path.abspath(os.path.dirname(__file__))

about = {}
with open(os.path.join(base_dir, "__about__.py")) as f:
    exec(f.read(), about)

# Get the long description from the relevant file, converting from md to rst if possible
with open(os.path.join(base_dir, 'README.md'), encoding='utf-8') as f:
    long_description = f.read()
try:
    from pypandoc import convert
    long_description = convert('README.md', 'rst', format='md')
except (ImportError, OSError) as e:
    print("warning: pypandoc module not found, could not convert Markdown to RST")

class MyBuild(build):
    def run(self):
        subprocess.call("make")
        build.run(self)

setup(name=about["__title__"],
      version=about["__version__"],
      description=about["__summary__"],
      long_description=long_description,
      author=about["__author__"],
      author_email=about["__email__"],
      url=about["__uri__"],
      license=about["__license__"],
      cmdclass={'build': MyBuild},
      data_files=[(get_python_lib() + '/', ['_pwrusb_swig_interface.so']),
                  (get_python_lib() + '/', ['libpowerusb.dylib'])],
      # See https://pypi.python.org/pypi?%3Aaction=list_classifiers
      classifiers=[
          'Environment :: MacOS X', # only intended for controlling pwrusb strips from Mac OSX
          'Operating System :: MacOS :: MacOS X', 
          
          'Topic :: Home Automation', 
          'Topic :: Scientific/Engineering', 
          
          # How mature is this project? Common values are
          #     Development Status :: 1 - Planning
          #     Development Status :: 2 - Pre-Alpha
          #     Development Status :: 3 - Alpha
          #     Development Status :: 4 - Beta
          #     Development Status :: 5 - Production/Stable
          #     Development Status :: 6 - Mature
          #     Development Status :: 7 - Inactive
          'Development Status :: 4 - Beta',

          # Indicate who your project is intended for
          'Intended Audience :: Science/Research',

          # Pick your license as you wish (should match "license" above)
          'License :: OSI Approved :: MIT License',

          # Specify the Python versions you support here. In particular, ensure
          # that you indicate whether you support Python 2, Python 3 or both.
  #         'Programming Language :: Python :: 2',
  #         'Programming Language :: Python :: 2.6', 
  #         'Programming Language :: Python :: 2.7',
  #         'Programming Language :: Python :: 3',
  #         'Programming Language :: Python :: 3.2',
          'Programming Language :: Python :: 3.3',
          'Programming Language :: Python :: 3.4',
          'Programming Language :: Python :: 3.5',
      ],
      keywords='pwrusb pdu usb power-control',
      py_modules=["pwrusb", "pwrusb_swig_interface"])

